/*
 * Programming in Java, Jacobs University, 2015
 *
 * This file demonstrates how to implement a simple chat client. It is
 * also used to emulate nasty clients in order to stress test chat
 * servers.
 */

/*
 * TODO:
 * - get rid of the BufferedReader()s since they tend to not react
 *   if, for example, the server goes down.
 */

import java.net.InetAddress;
import java.net.Socket;

import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.OutputStreamWriter;
import java.io.IOException;

public class ChatClient
    implements Runnable
{
    static final int PORT = 4321;

    private Socket socket;
    private BufferedReader in;
    private OutputStreamWriter out;

    public ChatClient(Socket s)
	throws IOException
    {
	socket = s;
	in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	out = new OutputStreamWriter(socket.getOutputStream());
    }

    public void sendln(String s)
	throws IOException
    {
	out.write(s, 0, s.length());
	out.write("\n", 0, 1);
	out.flush();
    }

    public void send(String s)
	throws IOException
    {
	out.write(s, 0, s.length());
	out.flush();
    }

    public void run()
    {
	try {
	    while (! Thread.currentThread().isInterrupted()) {
		String s = in.readLine();
		if (s == null) break;
		System.out.println(s);
	    }
	} catch (IOException e) {
	    System.err.println(e);
	} finally {
	    try {
		socket.close();
	    } catch (IOException ignored) { }
	}

	/*
	 * We need to tell the main thread that the game is over; one
	 * option is to close the system input but then this does not
	 * work since the buffered reader does not notice this
	 * immediately... So we do a System.exit() here but this is
	 * not really nice.
	 */
	
	try {
	    System.in.close();
	} catch (IOException e) {
	    System.err.println(e);
	}
	System.exit(0);
    }

    public static void main(String[] args)
    {
	try {
	    InetAddress addr = InetAddress.getByName("localhost");
	    Socket socket = new Socket(addr, ChatClient.PORT);
	    ChatClient c = new ChatClient(socket);
	    Thread t = new Thread(c);
	    t.start();
	    
	    BufferedReader in;
	    in = new BufferedReader(new InputStreamReader(System.in));
	    
	    String msg;
	    while ((msg = in.readLine()) != null) {
		try {
		    c.sendln(msg);
		} catch (IOException e) {
		    System.err.println(e);
		    break;
		}
	    }
	    
	    /*
	     * Interrupting the thread unfortunately does not lead to
	     * an exception while hanging in readLine() so we have to
	     * use a more drastic measure such as closing the
	     * underlying socket.
	     */
	    
	    t.interrupt();
	    socket.shutdownInput();
	} catch (IOException e) {
	    System.err.println(e);
	    System.exit(1);
	}
    }
}
